"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
parser for bcmcmd phy diag dsc/pcs
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "bcmcmd phy diag"
aliases = ['bcm phy diag', 'dsc swp']

# This has to handle 2 totally different outputs:
# bcm phy diag xe1 dsc
# bcm phy diag xe1 pcs

'''
2.x - strange behaviors, will probably ignore 2.x version:

root@act-5610-05:~# bcm phy diag xe12 dsc


DSC parameters for port 13

PLL Range: 234

LN  PPM PPM_XFR clk90_ofs PF SL_TRGT VGA BIAS DFE1 DFE2 DFE3 DFE4 DFE5 PREC MAIN POSTC MTRC PE   ZE   ME  PO  ZO  MO
00 0000 0000000 000000035 0010 0150 0047 100% 0000 0000 0000 0000 0000 0000 0045 0018  0000 0000 0000  0   0   1  0
01 0000 0000000 000000032 0010 0150 0047 100% 0000 0000 0000 0000 0000 0000 0045 0018  0000 0000 0000  0   0   0  0
02 0049 0000000 000000032 0010 0150 0047 100% 0000 0000 0000 0000 0000 0000 0063 0000  0000 0000 0000  0   0   0  0
03 0047 0000000 000000032 0010 0150 0047 100% 0000 0000 0000 0000 0000 0000 0063 0000  0000 0000 0000  0   0   0  0


SSH Session:
root@act-6701-11:~# /usr/lib/cumulus/bcmcmd phy xe26 dsc
Port xe26 (PHY addr 0x81) Reg 0x0000: 0x0000
root@act-6701-11:~#

Console (had link down/up messages interleaved):
root@act-6701-11:~# /usr/lib/cumulus/bcmcmd phy diag xe26 dsc

TSC-A2 uC ver=a042 crc=a081 T=37 T_idx=8 rev=6.4.0 0x7fbbd 0xfff3f13f 0xbf7bfeb3 0x5fe7defb

  L0 ext_det=1 (lh=0 ll=0)  afe_det=0 (lh=0 ll=0)  sig_ok=0 (lh=0 ll=0)  pmd_lck=0 (lh=0 ll=0)
  L1 ext_det=1 (lh=0 ll=0)  afe_det=0 (lh=0 ll=0)  sig_ok=0 (lh=0 ll=0)  pmd_lck=0 (lh=0 ll=0)
  L2 ext_det=1 (lh=0 ll=0)  afe_det=0 (lh=0 ll=0)  sig_ok=0 (lh=0 ll=0)  pmd_lck=0 (lh=0 ll=0)
  L3 ext_det=1 (lh=0 ll=0)  afe_det=0 (lh=0 ll=0)  sig_ok=0 (lh=0 ll=0)  pmd_lck=0 (lh=0 ll=0)

Tx Amps. Lane 0: IDriver:4 PreDriver:4 P2_Coeff:2
Tx_Taps, Lane:0: Pre:0x00 Main:0x27 Post:0x18
Tx Amps. Lane 1: IDriver:4 PreDriver:4 P2_Coeff:2
Tx_Taps, Lane:1: Pre:0x00 Main:0x27 Post:0x18
Tx Amps. Lane 2: IDriver:4 PreDriver:4 P2_Coeff:2
Tx_Taps, Lane:2: Pre:0x00 Main:0x27 Post:0x18
Tx Amps. Lane 3: IDriver:4 PreDriver:4 P2_Coeff:2
Tx_Taps, Lane:3: Pre:0x00 Main:0x27 Post:0x18


DSC parameters for port 27

PLL Range: 231(=0x00e7)

LN SD PMD(FW_MODE) PPM  PPM_XFR ck90_ofs | PF   p1_lvl m1  |VGA BIAS DFE1 2   3   |DFE4 5  TX_DRVR |PREC MAIN POSTC MTRC|  PE   ZE  ME  PO  ZO  MO |DSC_SM|ST
00 00 BRx1+DFE(0)  0000 0000000 000000037| 0008   150    0 |47  100% 00   000 000 |000 000 0x2440  |0000 0039 0024 +0000|   0    0   0   0   0   0 |0x0000|00
01 00 BRx1+DFE(0)  0000 0000000 000000038| 0008   150    0 |47  100% 00   000 000 |000 000 0x2440  |0000 0039 0024 +0000|   0    0   0   0   0   0 |0x0000|00
02 00 BRx1+DFE(0)  0000 0000000 000000037| 0008   150    0 |47  100% 00   000 000 |000 000 0x2440  |0000 0039 0024 +0000|   0    0   0   0   0   0 |0x0000|00
03 00 BRx1+DFE(0)  0000 0000000 000000037| 0008   150    0 |47  100% 00   000 000 |000 000 0x2440  |0000 0039 0024 +0000|   0    0   0   0   0   0 |0x0000|00

4610:
DSC parameters for port 4

junction temperature: core0 407c  

PLL Range: 7

LN SD PMD(FW_MODE)  PPM clk90_ofs|clkp1_of p1_lvl m1_lvl   PF lowp| VGA BIAS DFE1 DFE2 DFE3|DFE4 DFE5 TX_DRVR|PREC MAIN POSTC MTRC   PE|  ZE   ME   PO   ZO   MO|DSC_SM
 0  1  BRx1+DFE(0)    0         0|       0      0      0    0    0|   0 100%    0    0    0|   0    0   0x260|   1    0     0    0    0|   0    0    0    0    0|     0
 1  0  BRx1+DFE(0)    0         0|       0      0      0    0    0|   0 100%    0    0    0|   0    0       0|   0    0     0    0    0|   0    0    0    0    0|     0
 2  0  BRx1+DFE(0)    0         0|       0      0      0    0    0|   0 100%    0    0    0|   0    0       0|   0    0     0    0    0|   0    0    0    0    0|     0
 3  0  BRx1+DFE(0)    0         0|       0      0      0    0    0|   0 100%    0    0    0|   0    0       0|   0    0     0    0    0|   0    0    0    0    0|     0


Not sure where this one came from, not 4610:
LN (CDRxN  , UC_CFG,TX_RX_RST,STP) SD LCK RXPPM CLK90 CLKP1 PF(M,L) VGA DCO P1mV M1mV  DFE(1,2,3,4,5,6)       TXPPM  TXEQ(n1,m,p1,2,3,rpara)   EYE(L,R,U,D)  LINK_TIME
 0 (OSx1:x1, 0xc0,     0,0,    0)  1   1*-292    32     0     0,1    2  63   0    0   0,  0,  0,  0,  0,  0      0   10, 65,20, 0, 0, 5       0,  0,  0,  0    24.9
 1 (OSx1:x1, 0xc0,     0,0,    0)  1   0*   0    32     0    10,0   32  63   0    0   0,  0,  0,  0,  0,  0      0   10, 65,20, 0, 0, 5       0,  0,  0,  0     0.0
 2 (OSx1:x1, 0xc0,     0,0,    0)  1   0*-291    32     0     0,0    3  63   0    0   0,  0,  0,  0,  0,  0      0   10, 65,20, 0, 0, 5       0,  0,  0,  0     0.0
 3 (OSx1:x1, 0xc0,     0,0,    0)  1   1*-292    32     0     0,1    2  63   0    0   0,  0,  0,  0,  0,  0      0   10, 65,20, 0, 0, 5       0,  0,  0,  0    23.0


4610 as of BCM SDK 6.5.14:
DSC parameters for port 50

PLL Range: 234

LN  PPM PPM_XFR clk90_ofs PF SL_TRGT VGA BIAS DFE1 DFE2 DFE3 DFE4 DFE5 PREC MAIN POSTC MTRC PE   ZE   ME  PO  ZO  MO
00 0003 0000000 000000032 0010 0150 0047 100% 0000 0000 0000 0000 0000 0000 0063 0000  0000 0000 0000  0   0   0  0
01 0002 0000000 000000036 0000 0150 0036 100% 0000 0000 0000 0000 0000 0000 0055 0008  -1024 0000 -012 10   0 -14 -9
02 0002 0000000 000000036 0000 0150 0036 100% 0000 0000 0000 0000 0000 0000 0055 0008  -1024 0000 0012  6   0 -15 -11
03 0002 0000000 000000036 0000 0150 0036 100% 0000 0000 0000 0000 0000 0000 0055 0008  -1024 0000 -008  2   0 -12 11

(end of output)
root@act-4610p-53:~# bcm phy diag ge0 dsc


DSC parameters for port 1

junction temperature: core0 407c  

PLL Range: 8

LN SD PMD(FW_MODE)  PPM clk90_ofs|clkp1_of p1_lvl m1_lvl   PF lowp| VGA BIAS DFE1 DFE2 DFE3|DFE4 DFE5 TX_DRVR|PREC MAI
 0  1  BRx1+DFE(0)    0         0|       0      0      0    0    0|   0 100%    0    0    0|   0    0   0x260|   1    
 1  0  BRx1+DFE(0)    0         0|       0      0      0    0    0|   0 100%    0    0    0|   0    0       0|   0    
 2  0  BRx1+DFE(0)    0         0|       0      0      0    0    0|   0 100%    0    0    0|   0    0       0|   0    
 3  0  BRx1+DFE(0)    0         0|       0      0      0    0    0|   0 100%    0    0    0|   0    0       0|   0    



3.x outputs:
root@dell-s6000-22:~# bcm phy diag xe1 dsc

TSC-A2 uC ver=a042 crc=a081 T=43 T_idx=8 rev=6.4.0 0x7fbbd 0xfff3f13f 0xbf7bfeb3 0x5fe7defb

  L0 ext_det=1 (lh=0 ll=0)  afe_det=1 (lh=0 ll=0)  sig_ok=0 (lh=0 ll=0)  pmd_lck=0 (lh=0 ll=0)
  L1 ext_det=1 (lh=0 ll=0)  afe_det=1 (lh=0 ll=0)  sig_ok=0 (lh=0 ll=0)  pmd_lck=0 (lh=0 ll=0)
  L2 ext_det=1 (lh=0 ll=0)  afe_det=1 (lh=1 ll=1)  sig_ok=0 (lh=0 ll=0)  pmd_lck=0 (lh=0 ll=0)
  L3 ext_det=1 (lh=0 ll=0)  afe_det=1 (lh=0 ll=0)  sig_ok=0 (lh=0 ll=0)  pmd_lck=0 (lh=0 ll=0)

Tx Amps. Lane 0: IDriver:6 PreDriver:6 P2_Coeff:2
Tx_Taps, Lane:0: Pre:0x00 Main:0x2f Post:0x10
Tx Amps. Lane 1: IDriver:7 PreDriver:7 P2_Coeff:2
Tx_Taps, Lane:1: Pre:0x00 Main:0x2b Post:0x14
Tx Amps. Lane 2: IDriver:6 PreDriver:6 P2_Coeff:2
Tx_Taps, Lane:2: Pre:0x00 Main:0x2e Post:0x11
Tx Amps. Lane 3: IDriver:6 PreDriver:6 P2_Coeff:2
Tx_Taps, Lane:3: Pre:0x00 Main:0x2f Post:0x10


DSC parameters for port 2

PLL Range: 232(=0x00e8)

LN SD PMD(FW_MODE) PPM  PPM_XFR ck90_ofs | PF   p1_lvl m1  |VGA BIAS DFE1 2   3   |DFE4 5  TX_DRVR |PREC MAIN POSTC MTRC|  PE   ZE  ME  PO  ZO  MO |DSC_SM|ST
00 01 BRx1+DFE(1)  0000 0000000 000000032| 0008   150    0 |47  100% 00   000 000 |000 000 0xa660  |0000 0047 0016 +0000|   0    0   0   0   0   0 |0x0000|00
01 01 BRx1+DFE(1)  0000 0000000 000000032| 0008   150    0 |47  100% 00   000 000 |000 000 0xa770  |0000 0043 0020 +0000|   0    0   0   0   0   0 |0x0000|00
02 01 BRx1+DFE(1)  0000 0000000 000000032| 0008   150    0 |47  100% 00   000 000 |000 000 0xa660  |0000 0046 0017 +0000|   0    0   0   0   0   0 |0x0000|00
03 01 BRx1+DFE(1)  0000 0000000 000000032| 0008   150    0 |47  100% 00   000 000 |000 000 0xa660  |0000 0047 0016 +0000|   0    0   0   0   0   0 |0x0000|00

03 01 OSx1+DFE(2)  0017 0000000 000000041| 0013   150    0 |26   88% 25   -11 -04 |003 -04 0x2220  |0000 0045 0018 +0000|  -1    0  -4   4   1   3 |0x0000|0d
03 01 OSx1(1)      0030 0000000 000000036| 0000   150    0 |36  100% 00   000 000 |000 000 0x2550  |0000 0044 0019 -1024|   0   -3  11   0   8   0 |0x0000|0d
00 01 OSx1+DFE(2)  -029 0000000 000000042| 0004   200    0 |18   88% 14   -12 001 |001 -03 0x2550  |0000 0045 0018 +0000|   1  -15  12  -1   9   0 |0x0000|0d
00 01 OSx8.2(0)    -067 0000000 000000032| 0010   150    0 |47  100% 00   000 000 |000 000 0x0990  |0000 0063 0000 +0000|   0    0   0   0   0   0 |0x0000|0d

Trident3
root@qct-ix8-51:~# bcm phy diag xe0 dsc
 tscf16_phy_pmd_info_dump:512 type = 16384 laneMask  = 0x4, Address = 0xD1
SerDes type = falcon16_tsc
CORE RST_ST  PLL_PWDN  UC_ATV   COM_CLK   UCODE_VER  AFE_VER   LIVE_TEMP   AVG_TMON   RESCAL   VCO_RATE  ANA_VCO_RANGE  REFCLK_2x  PLL_DIV  PLL_LOCK
00    0,00      0        1     156.25MHz   D103_04     0xc0       41C      (16)41C    0x07    20.625GHz     207            0       132        1   


Tomahawk 100G

root@act-7712-04:~# bcm phy diag ce1 dsc
 tscf_phy_pmd_info_dump:497 type = 16384 laneMask  = 0xF, Address = 0xE9
SerDes type = falcon_tsc
CORE RST ST PLL_PWDN  UC_ATV   COM_CLK   UCODE_VER  AFE_VER   LIVE_TEMP   AVG_TMON   RESCAL   VCO_RATE  ANA_VCO_RANGE  PLL_DIV    PLL_LOCK
00   0   00    0        1     156.25MHz   D10B_13     0xb2       42C      (16) 41C    0x07    25.750GHz     222       (07) 165       1*

**************************************************************************************************************
****                          Legend of Entries in display_core_state()                                   ****
**************************************************************************************************************
*  RST              : Core DP Reset State{reset_active, reset_occured, reset_held}                           *
*  ST               : Core uC Status byte(hex)                                                               *
*  PLL_PWDN         : PLL Powerdown Control Bit (active high)                                                *
*  UC_ATV           : UC Active bit                                                                          *
*  COM_CLK          : COM Clock frequency in MHz                                                             *
*  UCODE_VER        : Microcode Version [majorversion_minorversion]                                          *
*  AFE_VER          : AFE Hardware Vesrion                                                                   *
*  LIVE_TEMP        : Live Die temperature in Celsius                                                        *
*  AVG_TMON         : uC Temp_idx, Average temperature in Celsius                                            *
*  RESCAL           : Analog Resistor Calibration value                                                      *
*  VCO_RATE         : uC VCO Rate in GHz (approximate)                                                       *
*  ANA_VCO_RANGE    : Analog VCO Range                                                                       *
*  PLL_DIV          : (Register Value) Actual PLL Divider Value                                              *
*  PLL_Lock         : PLL Lock                                                                               *
**************************************************************************************************************
LN (CDRxN      , UC_CFG,RST,STP)  SD LCK RXPPM CLK90 CLKP1 PF(M,L)  VGA DCO P1mV M1mV  DFE(1,2,3,4,5,6)        TXPPM TXEQ(n1,m,p1,2,3) TXAMP   EYE(L,R,U,D)  LINK_TIME
 0 (OSx1       , 0x0440,   0, 0)  0*  0     0    32     0   10,0    32   0    0    0   0,  0,  0,  0,  0,  0     0    0,100,12, 0, 0    8,0    0,  0,  0,  0     0.0
 1 (OSx1       , 0x0440,   0, 0)  0*  0     0    32     0   10,0    32   0    0    0   0,  0,  0,  0,  0,  0     0    0,100,12, 0, 0    8,0    0,  0,  0,  0     0.0
 2 (OSx1       , 0x0440,   0, 0)  0*  0     0    32     0   10,0    32   0    0    0   0,  0,  0,  0,  0,  0     0    0,100,12, 0, 0    8,0    0,  0,  0,  0     0.0
 3 (OSx1       , 0x0440,   0, 0)  0*  0     0    32     0   10,0    32   0    0    0   0,  0,  0,  0,  0,  0     0    0,100,12, 0, 0    8,0    0,  0,  0,  0     0.0

TD3:
LN (CDRxN  , UC_CFG,TX_RX_RST,STP) SD LCK RXPPM CLK90 CLKP1 PF(M,L) VGA DCO P1mV M1mV  DFE(1,2,3,4,5,6)       TXPPM  TXEQ(n1,m,p1,2,3,rpara)   EYE(L,R,U,D)  LINK_TIME
 0 (OSx2:x2, 0x40,     0,0,    0)  0   0    0    32     0    10,0   32   0   0    0   0,  0,  0,  0,  0,  0      0   10, 65,20, 0, 0, 5       0,  0,  0,  0     0.0

TD3 X5:
 2 (OSxT:xT, 0x0000, 0x00 , 7,7, 0) 1* 0    0     0     0     0,0   39   0   0    0   0,  0,  0,  0,  0,  0      0    5, 80,15, 0, 0, 0       0,  0,  0,   0    0.0
'''


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    pattern_hdr1 = re.compile(r'''^
    \s*TSC-(?P<tsc_ver>\w+)
    \s+uC\sver=(?P<microcode_ver>[a-z0-9]+)
    \s+crc=(?P<crc>[a-z0-9]+)
    \s+T=(?P<temperature>[0-9]+)
    \s+T_idx=(?P<t_idx>[0-9]+)
    \s+rev=(?P<rev>[0-9\.]+)
    \s+(?P<hex1>[x0-9a-f]+)
    \s+(?P<hex2>[x0-9a-f]+)
    \s+(?P<hex3>[x0-9a-f]+)
    \s+(?P<hex4>[x0-9a-f]+)
    \s*$''', re.X)

    pattern_data2_sub_text = r'''
    \s+ext_det=(?P<ext_detect>\d+)
    \s+\(lh=(?P<ext_detect_lh>\d+)
    \s+ll=(?P<ext_detect_ll>\d+)\)
    \s+afe_det=(?P<analog_front_end_detect>\d+)
    \s+\(lh=(?P<analog_front_end_detect_lh>\d+)
    \s+ll=(?P<analog_front_end_detect_ll>\d+)\)
    \s+sig_ok=(?P<signal_detected_ok>\d+)
    \s+\(lh=(?P<signal_detected_ok_lh>\d+)
    \s+ll=(?P<signal_detected_ok_ll>\d+)\)
    \s+pmd_lck=(?P<pmd_clock_recvry_lock>\d+)
    \s+\(lh=(?P<pmd_clock_recvry_lock_lh>\d+)
    \s+ll=(?P<pmd_clock_recvry_lock_ll>\d+)\)
    '''

    pattern_data2_sub = re.compile(pattern_data2_sub_text, re.X)
    pattern_data2 = re.compile(r'^\s*(?P<lane>L[0-4]+)(?P<submatch>%s)\s*$' % pattern_data2_sub_text, re.X)

    pattern_tx_amps_sub_txt = r'''
    \s*IDriver:(?P<idriver>\d+)
    \s+PreDriver:(?P<pre_driver>\d+)
    \s+P2_Coeff:(?P<p2_coeff>\d+)
    '''

    # pattern_tx_amps = re.compile(r'''^
    # \s*Tx\sAmps.\sLane\s(?P<lane>\d+):
    # \s+IDriver:(?P<idriver>\d+)
    # \s+PreDriver:(?P<pdriver>\d+)
    # \s+P2_Coeff:(P?<p2_coeff>\d+)
    # ''', re.X)

    pattern_tx_amps_sub = re.compile(pattern_tx_amps_sub_txt, re.X)
    pattern_tx_amps = re.compile(r'^\s*Tx\sAmps.\sLane\s(?P<lane>\d+):(?P<submatch>%s)\s*$' % pattern_tx_amps_sub_txt, re.X)

    pattern_tx_taps_sub_txt = r'''
    \s*Pre:(?P<pre_cursor_tap_coeff_hex>[x0-9a-f]+)
    \s+Main:(?P<main_tap_coeff_hex>[x0-9a-f]+)
    \s+Post:(?P<post_cursor_tap_coeff_hex>[x0-9a-f]+)
    '''

    # pattern_tx_taps = re.compile(r'''^
    # \s*Tx_Taps,\sLane:(?P<lane>\d+):
    # \s+Pre:(?P<pre>[x0-9a-f]+)
    # \s+Main:(?P<main>[x0-9a-f]+)
    # \s+Post:(?P<post>[x0-9a-f]+)
    # \s*$''', re.X)

    pattern_tx_taps_sub = re.compile(pattern_tx_taps_sub_txt, re.X)
    pattern_tx_taps = re.compile(r'^\s*Tx_Taps,\sLane:(?P<lane>\d+):(?P<submatch>%s)\s*$' % pattern_tx_taps_sub_txt, re.X)

    pattern_tscef_hdr = re.compile(r'''\s*tsc[ef16]+_phy_pmd_info_dump:[0-9]+\stype =\s[0-9]+\slaneMask\s+=\s(?P<lane_mask>0x[0-9A-F]+)(,\sAddress\s=\s0x[0-9A-F]+)?\s*$''')
    # tsce_phy_pmd_info_dump:458 type = 16384 laneMask  = 0x8
    # tscf_phy_pmd_info_dump:498 type = 16384 laneMask  = 0xF, Address = 0xA1

    pattern_blackhawk_hdr = re.compile(r'''\s*blackhawk_phy_pmd_info_dump:[0-9]+\stype =\s[0-9]+\slaneMask\s+=\s(?P<lane_mask>0x[0-9A-F]+)?\s*$''')
    # blackhawk_phy_pmd_info_dump:454 type = 16384 laneMask  = 0xC

    pattern_serdes_ignore = re.compile('''^\s*SerDes type = falcon[16]*_tsc\s*$''')
    pattern_junction_ignore = re.compile('''junction\s+temperature:\s+core[0123]\s+[0-9a-f]+''')

    pattern_dsc_hdr_ignore = re.compile(r'''^DSC\sparameters\sfor\sport\s\d+\s*$''')


    pattern_pll = re.compile(r'''^PLL\sRange:\s(?P<pll_range_dec>\d+)(\(=(?P<pll_range_hex>[x0-9a-f]+)\))?\s*$''')
    pattern_tscef_core_hdr_ignore = re.compile(r'''
        ^\s*CORE\s+RST[_\s]+ST\s+PLL_PWDN\s+UC_ATV
        \s+COM_CLK\s+UCODE_VER(\s+API_VER)?
        \s+AFE_VER\s+LIVE_TEMP\s+AVG_TMON
        \s+RESCAL\s+VCO_RATE\s+ANA_VCO_RANGE
        (\s+REFCLK_2x)?\s+PLL_DIV\s+PLL_LOCK(\s+PLL_COMP_THRESH)?$
        ''', re.X)
    pattern_tscef_core_data = re.compile(r'''
    ^\s*(?P<core>[0-9A-F]+)
    \s+(?P<core_reset>[0-9]+)
    [,\s]+(?P<core_status>[0-9A_F]+)
    \s+(?P<pll_powerdown_bit>[01])
    \s+(?P<uc_active_bit>[01])
    \s+(?P<com_clock_freq>[0-9\.]+MHz)
    \s+(?P<ucode_ver>[0-9A-F\_]+)
   (\s+(?P<api_ver>[0-9A-F\_]+))?
    \s+(?P<afe_ver>0x[0-9a-f]+)
    \s+(?P<live_temp>[\-0-9]+C)
    \s+(?P<avg_temp>[\-0-9\(\)\s]+C)
    \s+(?P<resistor_calibr>0x[0-9a-f]+)
    \s+(?P<vco_rate>[0-9\.]+GHz)
    \s+(?P<analog_vco_range>[0-9]+)
    \s+(?P<refclock_double>[01])?
    (\s+(?P<pll_register>\([0-9]+\)))?
    \s+(?P<pll_divider>[0-9]+)
    \s+(?P<pll_lock>[01]\*?)
   (\s+(?P<pll_lock_threshold>[0-9mV-]+))?\s*$
    ''', re.X)

    pattern_hdr4_ignore = re.compile(r'''^
    LN\s+SD\s+PMD\(FW_MODE\)\s+PPM\s+PPM_XFR\s+ck90_ofs
    \s+|\s+PF\s+p1_lvl\sm1
    \s+|VGA\s+BIAS\s+DFE1\s+2\s+3\s+|DFE4\s+5\s+TX_DRVR
    \s+|PREC\s+MAIN\s+POSTC\s+MTRC
    \s*|\s+PE\s+ZE\s+ME\s+PO\s+ZO\s+MO
    \s+|DSC_SM
    \s+|ST
    \S*$''', re.X)

    pattern_helix4_ge_hdr = re.compile(r'''
    ^LN\s+SD\s+PMD\(FW_MODE\)\s+PPM\s+clk90_ofs\|clkp1_of\s+p1_lvl\s+m1_lvl\s+PF\s+lowp\|\s+
    VGA\s+BIAS\s+DFE1\s+DFE2\s+DFE3\|DFE4\s+DFE5\s+TX_DRVR\|PREC\s+MAIN\s+
    POSTC\s+MTRC\s+PE\|\s+ZE\s+ME\s+PO\s+ZO\s+MO\|DSC_SM
    ''', re.X)

    pattern_helix4_ge_channel_data = re.compile(r'''
    \s*(?P<lane>[0123])
    \s+(?P<signal_detect>[0-9]+)
    \s+(?P<pmd_clock_and_data_recovery_mode>BR|OS)x
    (?P<pmd_val>[\.0-9]+)
    (\+(?P<dfe>DFE))?
    \((?P<pmd_firmware_mode>[0-9]+)\)
    \s+(?P<ppm>[\-0-9]+)
    \s+(?P<ck90_ofs>[0-9]+)\|
    \s+(?P<clkp1_diag_slicer_delay>[\-0-9]+)
    \s+(?P<p1_lvl>[0-9]+)
    \s+(?P<m1_lvl>[0-9]+)
    \s+(?P<pf_peaking_filter>[0-9]+)
    \s+(?P<low_p>[0-9]+)\|
    \s+(?P<vga_variable_gain_amplifier>[0-9]+)
    \s+(?P<bias>[0-9]+)%
    \s+(?P<dfe_decision_feedback_equalizer_1>[0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_2>-?[0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_3>-?[0-9]+)\|
    \s+(?P<dfe_decision_feedback_equalizer_4>-?[0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_5>-?[0-9]+)
    \s+(?P<tx_drvr_amplitude>[x0-9a-f]+)\|
    \s+(?P<pre_cursor_tap_coeff_dec>[0-9]+)
    \s+(?P<main_tap_coeff_dec>[0-9]+)
    \s+(?P<post_cursor_tap_coeff_dec>[0-9]+)
    \s+(?P<mtrc>[\+\-0-9]+)
    \s+(?P<pe>-?[0-9]+)\|
    \s+(?P<ze>-?[0-9]+)
    \s+(?P<me>-?[0-9]+)
    \s+(?P<po>-?[0-9]+)
    \s+(?P<zo>-?[0-9]+)
    \s+(?P<mo>-?[0-9]+)\|
    \s+(?P<dsc_sm>[x0-9a-f]+)
    ''', re.X)

    pattern_helix4_sfp_channel_hdr = re.compile(r'''
    ^LN\s+PPM\s+PPM_XFR\s+clk90_ofs\s+PF\s+SL_TRGT\s+VGA\s+BIAS\s+
    DFE1\s+DFE2\s+DFE3\s+DFE4\s+DFE5\s+PREC\s+MAIN\s+POSTC\s+MTRC\s+PE\s+ZE\s+ME\s+PO\s+ZO\s+MO\s*$
    ''', re.X)

    pattern_helix4_sfp_channel_data = re.compile(r'''
    \s*(?P<lane>0?[0123])
    \s+(?P<ppm>[\-0-9]+)
    \s+(?P<ppm_xfer>[0-9]+)
    \s+(?P<ck90_ofs>[0-9]+)
    \s+(?P<pf_peaking_filter>[0-9]+)
    \s+(?P<sl_target>[0-9]+)
    \s+(?P<vga_variable_gain_amplifier>[0-9]+)
    \s+(?P<bias>[0-9]+)%
    \s+(?P<dfe_decision_feedback_equalizer_1>[\-0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_2>[\-0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_3>[\-0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_4>[\-0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_5>[\-0-9]+)
    \s+(?P<pre_cursor_tap_coeff_dec>[0-9]+)
    \s+(?P<main_tap_coeff_dec>[0-9]+)
    \s+(?P<post_cursor_tap_coeff_dec>[0-9]+)
    \s+(?P<mtrc>[\+\-0-9]+)
    \s+(?P<slicer_pe>[\-0-9]+)
    \s+(?P<slicer_ze>[\-0-9]+)
    \s+(?P<slicer_me>[\-0-9]+)
    \s+(?P<slicer_po>[\-0-9]+)
    \s+(?P<slicer_zo>[\-0-9]+)
    \s+(?P<slicer_mo>[\-0-9]+)
    ''', re.X)

    pattern_tscef_channel_hdr = re.compile(r'''
    ^LN\s+\(CDRxN\s+,\s+UC_CFG,(UC_STS,)?(TX_RX_)?RST,STP\)
    \s+SD\s+LCK\s+RXPPM\s+CLK90\s+CLKP1\s+PF\(M,L\)
    \s+VGA\s+DCO\s+P1mV(\s+M1mV)?\s+DFE\(1,2,3,4,5,(6)?(dcd1,dcd2)?\)
    (\s+SLICER\(ze,zo,pe,po,me,mo\))?
    \s+TXPPM\s+TXEQ\(n1,m,p1,2,3(,rpara)?\)(\s+TXAMP)?
    \s+EYE\(L,R,U,D\)\s+LINK_TIME(\s+BER)?
    \s*$
    ''', re.X)

    pattern_tscef_50g_skip_lanes_ignore = re.compile(r'''
    ^\s*\-\s*skip\slane=[0-3]\s*$
    ''', re.X)

    pattern_tscef_channel_data = re.compile(r'''
    ^\s*(?P<lane>[0123])
    \s+\((?P<cdr_type>OSx[0-9a-zA-Z\.:]+)\s*,
    \s+(?P<microlane_cfg>0x[0-9a-f]+),
    (\s+(?P<microlane_sts>0x[0-9a-f]+)\s*,)?
    \s+(?P<reset_state>[0-9]),
    (?P<reset_state_rx>[0-9],)?
    \s+(?P<stopped_state>[01])\)
    \s+(?P<signal_detect>[01])\*?
    \s+(?P<pmd_rx_lock>[01])\*?
    \s*(?P<rx_clk_freq_offset>[\-0-9]+)
    \s+(?P<clk90_m1_slicer_delay>[\-0-9]+)
    \s+(?P<clkp1_diag_slicer_delay>[\-0-9]+)
    \s+(?P<peaking_filter_main>[0-9]+),
    (?P<peaking_filter_lowfreq>[0-9]+)
    \s+(?P<vga_variable_gain_amplifier>[0-9]+)
    \s+(?P<dc_offset_dac_ctrl>[\-0-9]+)
    \s+(?P<p1_slicer_vert_thresh>[\-0-9]+)
    (\s+(?P<m1_slicer_vert_thresh>[\-0-9]+))?
    \s+(?P<dfe_decision_feedback_equalizer_1>[\-0-9]+),
    \s*(?P<dfe_decision_feedback_equalizer_2>[\-0-9]+),
    \s*(?P<dfe_decision_feedback_equalizer_3>[\-0-9]+),
    \s*(?P<dfe_decision_feedback_equalizer_4>[\-0-9]+),
    \s*(?P<dfe_decision_feedback_equalizer_5>[\-0-9]+),
    (\s*(?P<dfe_decision_feedback_equalizer_6>[\-0-9]+))?
    (\s*(?P<dfe_decision_feedback_equalizer_dcd1>[\-0-9]+),
    \s*(?P<dfe_decision_feedback_equalizer_dcd2>[\-0-9]+))?
    
    (\s+(?P<slicer_ze>[\-0-9]+),
    \s*(?P<slicer_zo>[\-0-9]+),
    \s*(?P<slicer_pe>[\-0-9]+),
    \s*(?P<slicer_po>[\-0-9]+),
    \s*(?P<slicer_me>[\-0-9]+),
    \s*(?P<slicer_mo>[\-0-9]+))?

    \s+(?P<tx_clk_freq_offset>[0-9]+)
    \s+(?P<tx_equalization_n1>[0-9]+),
    \s*(?P<tx_equalization_m>[0-9]+),
    \s*(?P<tx_equalization_p1>[0-9]+),
    \s*(?P<tx_equalization_p2>[0-9]+),
    \s*(?P<tx_equalization_p3>[0-9]+)
    (,\s*(?P<tx_equalization_rpara>[0-9]+))?
    (\s+(?P<tx_amp1>[0-9]+),(?P<tx_amp2>[0-9]+))?
    \s+(?P<eye_left>[0-9]+),
    \s*(?P<eye_right>[0-9]+),
    \s*(?P<eye_up>[0-9]+),
    \s*(?P<eye_down>[0-9]+)
    \s+(?P<link_time>[0-9\.\>]+)
    (\s+(?P<bit_error>[0-9a-zA-Z\.!]+))?
    \s*$''', re.X)

    pattern_data4_sub_txt = r'''
    (?P<signal_detect>[0-9]+)
    \s+(?P<pmd_clock_and_data_recovery_mode>BR|OS)x
    (?P<pmd_val>[\.0-9]+)
    (\+(?P<dfe>DFE))?
    \((?P<pmd_firmware_mode>[0-9]+)\)
    \s+(?P<ppm>[\-0-9]+)
    \s+(?P<ppm_xfer>[0-9]+)
    \s+(?P<ck90_ofs>[0-9]+)\|
    \s+(?P<pf_peaking_filter>00[0-9]+)
    \s+(?P<p1_lvl>[0-9]+)
    \s+(?P<m1>[0-9]+)
    \s+\|(?P<vga_variable_gain_amplifier>[0-9]+)
    \s+(?P<bias>[0-9]+)%
    \s+(?P<dfe_decision_feedback_equalizer_1>[0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_2>-?[0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_3>-?[0-9]+)
    \s+\|(?P<dfe_decision_feedback_equalizer_4>-?[0-9]+)
    \s+(?P<dfe_decision_feedback_equalizer_5>-?[0-9]+)
    \s+(?P<tx_drvr_amplitude>[x0-9a-f]+)
    \s+\|(?P<pre_cursor_tap_coeff_dec>[0-9]+)
    \s+(?P<main_tap_coeff_dec>[0-9]+)
    \s+(?P<post_cursor_tap_coeff_dec>[0-9]+)
    \s+(?P<mtrc>[\+\-0-9]+)\|
    \s+(?P<pe>-?[0-9]+)
    \s+(?P<ze>-?[0-9]+)
    \s+(?P<me>-?[0-9]+)
    \s+(?P<po>-?[0-9]+)
    \s+(?P<zo>-?[0-9]+)
    \s+(?P<mo>-?[0-9]+)
    \s+\|(?P<dsc_sm>[x0-9a-f]+)
    \|(?P<st>[0-9a-f]+)
    '''

    pattern_data4_sub = re.compile(pattern_data4_sub_txt, re.X)
    pattern_data4 = re.compile(r'^(?P<lane>\d+)\s+(?P<submatch>%s)\s*$' % pattern_data4_sub_txt, re.X)

    pattern_legend_state = re.compile(r'''^\*.*$''')
    pattern_lane_state = re.compile(r'''\*+\s+Legend of Entries in display_lane_state\(\)\s+\*+$''')
    parsed_dct = {}

    # There are around 4 different possible segments in the non-Tomahawk output and some one-liners
    # 1) TSC-A2 uC ver=a042 crc=a081 T=43 T_idx=8 rev=6.4.0 0x7fbbd 0xfff3f13f 0xbf7bfeb3 0x5fe7defb
    # 2)   L0 ext_det=1 (lh=0 ll=0)  afe_det=0 (lh=0 ll=0)  sig_ok=0 (lh=0 ll=0)  pmd_lck=0 (lh=0 ll=0)
    #   [ repeated for L1-L3]
    # 3) Tx Amps. Lane 0: IDriver:6 PreDriver:6 P2_Coeff:2
    #    Tx_Taps, Lane:0: Pre:0x00 Main:0x2f Post:0x10
    #   [ repeated for L1-L3 ]
    # 4) LN SD PMD(FW_MODE) PPM  PPM_XFR ck90_ofs | PF   p1_lvl m1  |VGA BIAS DFE1 2   3   |DFE4 5  TX_DRVR |PREC MAIN POSTC MTRC|  PE   ZE  ME  PO  ZO  MO |DSC_SM|ST
    #    00 01 BRx1+DFE(1)  0000 0000000 000000032| 0008   150    0 |47  100% 00   000 000 |000 000 0xa660  |0000 0047 0016 +0000|   0    0   0   0   0   0 |0x0000|00
    #   [ repeated for 01-03 ]
    # Note, this appears for 10G ports (one lane) and 40G ports (four lanes), so for 10G, it could be looking
    #  at the first 4 ports at once

    helix4_sfp = False
    helix4_ge = False

    for line in output.split('\n'):
        debug('Line: {}'.format(line))

        if re.match('^\s*$', line):
            # blank line, skip
            continue

        match1 = pattern_hdr1.match(line)
        if match1:
            debug('Match: ' + pformat(match1.group(0)))
            # There is only one line that matches this segment
            parsed_dct['segment1'] = match1.groupdict()
            # parsed_dct['segment1'] = line_match(pattern_hdr1, line)
            continue

        match2 = pattern_data2.match(line)
        if match2:
            debug('Match: ' + pformat(match2.group(0)))
            # There will be four lines that match this segment
            if 'segment2' not in parsed_dct:
                parsed_dct['segment2'] = {}
            lane = match2.group('lane')
            lane = re.sub('L', 'Lane', lane)
            # parsed_dct['segment2'][lane] = line_match(pattern_data2_sub, match2.group('submatch'))
            # match2_sub is already included in match2, so it has to match again.
            match2_sub = pattern_data2_sub.match(match2.group('submatch'))
            parsed_dct['segment2'][lane] = match2_sub.groupdict()
            continue

        match3_amps = pattern_tx_amps.match(line)
        if match3_amps:
            # There will be 4 sets of tx_amps/tx_taps. one for each lane
            if 'segment3' not in parsed_dct:
                parsed_dct['segment3'] = {'tx_amps': {}, 'tx_taps': {}}
            lane = match3_amps.group('lane')
            lane = 'Lane' + lane
            match3_amps_sub = pattern_tx_amps_sub.match(match3_amps.group('submatch'))
            parsed_dct['segment3']['tx_amps'][lane] = match3_amps_sub.groupdict()
            continue

        match3_taps = pattern_tx_taps.match(line)
        if match3_taps:
            # There will be 4 sets of tx_amps/tx_taps. one for each lane
            if 'segment3' not in parsed_dct:
                parsed_dct['segment3'] = {'tx_amps': {}, 'tx_taps': {}}
            lane = match3_taps.group('lane')
            lane = 'Lane' + lane

            match3_taps_sub = pattern_tx_taps_sub.match(match3_taps.group('submatch'))
            parsed_dct['segment3']['tx_taps'][lane] = match3_taps_sub.groupdict()
            continue

        if pattern_dsc_hdr_ignore.match(line):
            continue

        if pattern_helix4_sfp_channel_hdr.match(line):
            helix4_sfp = True
            parsed_dct['lane_state'] = {}
            continue

        if helix4_sfp:
            match_helix4_sfp_channel = pattern_helix4_sfp_channel_data.match(line)
            if match_helix4_sfp_channel:
                # There will be 4 lines that match this section.
                helix4_sfp_lane = match_helix4_sfp_channel.group('lane')
                parsed_dct['lane_state'][helix4_sfp_lane] = match_helix4_sfp_channel.groupdict()
                # remove the higher key from the lower level
                parsed_dct['lane_state'][helix4_sfp_lane].pop('lane', 0)
                continue

        match_tscef_hdr = pattern_tscef_hdr.match(line)
        if match_tscef_hdr:
            parsed_dct['lane_mask'] = match_tscef_hdr.groupdict()['lane_mask']
            continue

        match_blackhawk_hdr = pattern_blackhawk_hdr.match(line)
        if match_blackhawk_hdr:
            parsed_dct['lane_mask'] = match_blackhawk_hdr.groupdict()['lane_mask']
            #No eye data is available. No need to parse additional lines
            break

        if pattern_serdes_ignore.match(line):
            continue

        if pattern_junction_ignore.match(line):
            continue

        if pattern_tscef_50g_skip_lanes_ignore.match(line):
            continue

        if pattern_tscef_core_hdr_ignore.match(line):
            continue

        if pattern_tscef_channel_hdr.match(line):
            continue

        if pattern_helix4_ge_hdr.match(line):
            helix4_ge = True
            parsed_dct['lane_state'] = {}
            continue

        if helix4_ge:
            match_helix4_ge_channel = pattern_helix4_ge_channel_data.match(line)
            if match_helix4_ge_channel:
                # There will be 4 lines that match this section.
                helix4_ge_lane = match_helix4_ge_channel.group('lane')
                parsed_dct['lane_state'][helix4_ge_lane] = match_helix4_ge_channel.groupdict()
                # remove the higher key from the lower level
                parsed_dct['lane_state'][helix4_ge_lane].pop('lane', 0)
                continue

        match2 = pattern_tscef_core_data.match(line)
        if match2:
            debug('Match: ' + pformat(match2.group(0)))
            # There is only one line that matches this segment
            parsed_dct['tscef_core'] = match2.groupdict()
            continue

        match_pll = pattern_pll.match(line)
        if match_pll:
            debug('Match: ' + pformat(match_pll.group(0)))
            # There is only one line that matches this segment
            parsed_dct['pll_range'] = match_pll.groupdict()
            continue

        if pattern_hdr4_ignore.match(line):
            continue

        match4 = pattern_data4.match(line)
        if match4:
            debug('Match: ' + pformat(match4.group(0)))
            # There will be four lines that match this segment
            if 'lane_state' not in parsed_dct:
                parsed_dct['lane_state'] = {}
            lane = match4.group('lane')
            lane = re.sub('^0', 'Lane', lane)
            match4_sub = pattern_data4_sub.match(match4.group('submatch'))
            parsed_dct['lane_state'][lane] = match4_sub.groupdict()
            continue

        match_tscef = pattern_tscef_channel_data.match(line)
        if match_tscef:
            # There will be four lines that match this segment
            if 'lane_state' not in parsed_dct:
                parsed_dct['lane_state'] = {}
            tscef_lane = match_tscef.group('lane')
            parsed_dct['lane_state'][tscef_lane] = match_tscef.groupdict()
            # also remove that higher_key from the lower level:
            parsed_dct['lane_state'][tscef_lane].pop('lane', 0)
            continue

        # Ignore Legends
        if pattern_lane_state.match(line):
            debug('last legend, just break out and end parsing')
            break

        if pattern_legend_state.match(line):
            # This one has to follow the previous legend, or this will break that one
            continue

        # Hit a line with no match.  crash out
        error('No {} parser match: {}'.format(name, line))
        return None

    return parsed_dct
