"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
Parser for bcmcmd phy dumpall c45
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "bcmcmd phy dumpall c45"
aliases = ['bcm phy dumpall c45']


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    pattern = re.compile(r'''^
     \s+(?P<PRTAD>0x[0-9A-F]+)
     \s+(?P<DEVAD>0x[0-9A-Fa-f]+)
     \s+(?P<REG>0x[0-9A-Fa-f]+):
     \s+(?P<VALUE>0x[0-9A-Fa-f]+)
     \s*$
    ''', re.X)

    parsed_dct = {}
    for line in output.split('\n'):
        debug('Line: {}'.format(line))
        # Throw away header line
        if re.search('PRTAD', line):
            debug('Dumping first line: {}'.format(line))
            continue
        match = pattern.match(line)

        if match:
            # Use 'PRTAD' as the primary key and 'REG' as the secondary key with value 'VALUE'
            debug('Match: ' + pformat(match.group(0)))
            # Capitalize the hex part, but not the 'x'
            prtad = 'x'.join(i.upper() for i in match.group('PRTAD').split('x'))
            devad = 'x'.join(i.upper() for i in match.group('DEVAD').split('x'))
            reg   = 'x'.join(i.upper() for i in match.group('REG').split('x'))
            value = 'x'.join(i.upper() for i in match.group('VALUE').split('x'))
            if prtad not in parsed_dct:
                parsed_dct[prtad] = {}
            if devad not in parsed_dct[prtad]:
                parsed_dct[prtad][devad] = {}
            parsed_dct[prtad][devad][reg] = value

        else:
            error('No {} parser match: {}'.format(name, output))
            return None

    return parsed_dct
