"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
parser for bcmcmd port
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "bcmcmd port"
aliases = ['bcm port']


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """
    # PORT: Status (* indicates PHY link up)
    # *xe46            LS(SW) Forced(10GFD) Stad(44:38:39:00:26:c9) STP(Disable) Lrn(ARL,FWD) UtPri(0) Pfm(FloodNone) IF(XGMII) PH(Slave) Max_frame(1518) MDIX(ForcedNormal, Normal) Medium(None) VLANFILTER(3)
    #  xe47            LS(SW) Forced(10GFD) Stad(44:38:39:00:26:ca) STP(Disable) Lrn(ARL,FWD) UtPri(0) Pfm(FloodNone) IF(XGMII) PH(Slave) Max_frame(1518) MDIX(ForcedNormal, Normal) Medium(None) Fault(Local) VLANFILTER(3)
    #  xe48   DISABLED LS(SW) Forced(40GFD) Stad(44:38:39:00:26:cb) STP(Disable) Lrn(FWD) UtPri(0) Pfm(FloodNone) IF(XLAUI) PH(Slave) Max_frame(1518) MDIX(ForcedNormal, Normal) Medium(None) Fault(LocalRemote) VLANFILTER(3)
    #  xe2             LS(SW) Forced(40GFD) Stad(70:72:cf:be:46:e8) STP(Disable) Lrn(FWD) UtPri(0) Pfm(FloodNone) IF(XGMII) Max_frame(1518) MDIX(ForcedNormal, Normal) Medium(Copper) Fault(Local) VLANFILTER(3)
    # *xe31    (EnabledLS(SW Forced((10GFD) Stad(c4:54:44:bd:02:f1) STP(Disable) Lrn(FWD)  UtPri(0) Pfm(FloodNone) IF(SFI) Max_frame(1518) MDIX(ForcedNormal, Normal)  Medium(Copper) Fault(Local) VLANFILTER(3)
    #  ge48    (Disabled LS(SW Forced((1GFD,pause) STP(Disable) Lrn(ARL,FWD) UtPri(0) Pfm(FloodNone) IF(GMII) PH(Slave) Max_frame(16356) MDIX(ForcedNormal, Normal) Medium(None) VLANFILTER(2) Mac driver(XMAC Driver)
    # *hg0     (EnabledLS(SW HIGIG2((12.7GFD) STP(Forward) IF(XGMII) Max_frame(16360) MDIX(ForcedNormal, Normal) Medium(Copper) VLANFILTER(3) Mac driver(XLMAC Driver)

    # re.X allows ignore of whitespace
    # force match on entire output, not ignore any new leading or trailing fields via '^' and '\s*$'
    pattern = re.compile(r"""^
                         \s+(?P<Up>\*?)
                         (?P<port>[cgxh][eg]\d+)
                         (\s+(?P<Disabled>DISABLED|\(Disabled))?
                         \s* 
                         (\(Enabled)? 
                         (?P<LS>LS)? 
                         \( 
                         (?P<SW>SW)
                         \)?
                         (\s+HIGIG2?)?
                         (\s+(?P<Auto>Auto))?
                         (\s+(?P<Forced>Forced))?
                         
                         \(\(?  
                         (?P<Speed>[0-9\.]+[GM]|no\slink)? 
                         (?P<Duplex>[FH]D)? 
                         ,?(?P<Pause>pause)?   
                         \) 
                         
                         (\s+Ability\s*\((?P<Ability>[,=_\s0-9A-Za-z]+)\))?
                         (Local\s*\((?P<Local>[,=_\s0-9A-Za-z]+)\))?
                         (Remote\s*\((?P<Remote>[,=_\s0-9A-Za-z]+)\))?

                         (\s*Stad\((?P<Stad>[0-9a-f:]+)\))?
                         \s+STP\((?P<STP>(Disable|Enable|Forward))\)
                         (\s+Lrn\((?P<Lrn>(ARL)?,?(FWD)?)\))?
                         (\s+UtPri\((?P<UtPri>\d+)\))?
                         (\s+Pfm\((?P<Pfm>\w+)\))?
                         \s+IF\((?P<IF>\w+)\)
                         (\s+PH\((?P<PH>\w+)\))?
                         (\s+LB\((?P<LB>\w+)\))?
                         \s+Max_frame\((?P<Max_frame>\d+)\)
                         \s+MDIX\((?P<MDIX>[a-zA-Z, ]+)\)
                         \s+Medium\((?P<Medium>\w+)\)
                         (\s+Fault\((?P<Fault>[a-zA-Z]+)\))?
                         (\s+VLANFILTER\((?P<VLANFILTER>[0-9]+)\))?
                         (\s+Mac\sdriver\((?P<Mac_driver>[\w ]+)\))?
                         \s*$
                         """, re.X)

    '''


                       \s+Lrn\(((ARL)?,?(FWD)?)\)
                                              \s+MDIX\((\w,?[ ]?\w?)\)

                       \s+Medium\((\w+)\)

                       \s+(Fault)?\(?(Local)?(Remote)?\)?
                       '''

    '''
    Move These to Tests
xe32
LS(SW)
Auto(no link)
Ability (fd = 10MB,100MB,1000MB,2500MB,5000MB,10GB hd =
intf = xgmii medium = copper pause = pause_tx,pause_rx,pause_asymm lb = none,MAC,PHY
flags = autoneg )Local (fd = 1000MB,10GB hd =  intf = medium =  pause =
pause_tx,pause_rx lb =  flags =  )
Stad(34:17:eb:f6:ea:e5)
STP(Disable)
Lrn(FWD)
UtPri(0)
Pfm(FloodNone)
IF(GMII)
Max_frame(1518)
MDIX(ForcedNormal, Normal)
    '''
    '''
act-5610-05, No Match found on line:
xe27
LS(SW)
Auto(no link)
Ability (fd = 100MB,1000MB,2500MB,10GB hd = 100MB intf =
gmii,sgmii,xgmii medium = fiber pause = pause_tx,pause_rx,pause_asymm lb =
none,MAC,PHY flags = autoneg )Local (fd = 1000MB,2500MB,10GB hd =  intf = medium =
pause = pause_tx,pause_rx lb =  flags =  )
Stad(cc:37:ab:32:fb:74)
STP(Disable)
Lrn(FWD)
UtPri(0)
Pfm(FloodNone)
IF(GMII)
Max_frame(1518)
MDIX(ForcedNormal, Normal)
Medium(Copper)
Fault(Local)
VLANFILTER(3)
    '''

    ''' Fields:
    ^\s
    (\*?)
    (x|ge[\d+])
    LS\((\w+)\)
    Forced\(([142][05]0?G|M[FH]D)\)
    Stad\(([0-9A-F:]+)\)
    STP\((Dis|En)able)\)
    Lrn\(((ARL)?,?(FWD)?)\)
    UtPri\((\d+)\)
    Pfm((\w+)\)
    IF\((\w+)\)
    PH\((\w+)\)
    Max_frame\((\d+)\)
    MDIX\((\w,?\w?\)\)
    Medium(\w+)
    (Fault)?\)?(Local)?(Remote)?\)?
    (VLANFILTER)?\(\d+\)
    '''
    # pattern=re.compile('^\s(\*?)(\w+)\s((DISABLED)?) LS((SW)?)\sForced([0-9]+[A-Z]+')
    # debug('Line: ' + line)

    parsed_dct = {}

    for line in output.split('\n'):
        if re.search('PORT', line):
            continue
        match = pattern.match(line)
        if match:
            debug('Match: ' + pformat(match.group(0)))
            port = match.group('port')
            parsed_dct[port] = match.groupdict()
            parsed_dct[port].pop('port', 0)
        else:
            error('Parser Bug: No Match found on line:')
            error(line)
            return None

    return parsed_dct
