"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import logging
import re
from pprint import pformat

"""
parser for bcmcmd show counters
"""

this_log = logging.getLogger(__name__)
debug = this_log.debug
info = this_log.info
warn = this_log.warning
error = this_log.error

name = "bcmcmd show counters"


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """
    bcm_show_counters_dct = {}
    named_groups = ['total', 'delta', 'rate']

    # pattern1=re.compile(r"""(?P<counter_name>[a-zA-Z0-9\_\(\)\.]+)
    #                     \s+:
    #                     \s+(?P<total>[0-9\,]+)
    #                     \s+(?P<delta>[\+0-9\,\+]+)
    #                     \s*(?P<rate>[0-9\,\+]+/s)?
    #                     """, re.X)
    pattern = re.compile(r"""(?P<counter_name>[a-zA-Z0-9_\(\)]+)
                         \.?
                         (?P<phy>[a-zA-Z0-9]+)?
                         (\s+)?:
                         \s+(?P<total>[0-9\,]+)
                         \s+(?P<delta>[\+0-9\,\+]+)
                         \s*(?P<rate>[0-9\,\+]+/s)?
                         """, re.X)

    note_pattern = re.compile(r'''\s*NOTE:\scounter\scollection    #Throw away this NOTE line
                              ''', re.X)
    # rq_drop_pattern = re.compile(r'''RQ_DROP ''')

    for line in output.split('\n'):
        if not line.strip():
            continue
        match = pattern.match(line)
        if match:
            debug('Matches: Countername: %s, Phy: %s, Total: %s, Delta: %s, Rate: %s' %
                  (match.group('counter_name'), match.group('phy'),
                   match.group('total'),
                   match.group('delta'), match.group('rate')))
            if match.group('phy'):
                # There can be a group of global RX_DROP counters (no phy).  I am skipping them for now
                if match.group('phy') not in bcm_show_counters_dct:
                    bcm_show_counters_dct[match.group('phy')] = {}
                bcm_show_counters_dct[match.group('phy')][match.group('counter_name')] = {}

                for name in named_groups:
                    # debug('Getting match value for %s' % name)
                    if match.group(name):
                        bcm_show_counters_dct[match.group('phy')][match.group('counter_name')][name] = (
                            int((match.group(name)).translate(None, ' +,/s')))
        else:
            if not note_pattern.match(line):
                warn('bcm counter output line did not match:\n %s' % line)

    debug('bcm_show_counters_dct:\n' + pformat(bcm_show_counters_dct))
    return bcm_show_counters_dct
