"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import logging
import re
from pprint import pformat

"""
parser for ethtool swpXX
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "ethtool"


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    # Use regex named groups. Always use \s instead of ' ' for spaces in output
    pattern = re.compile(r'''
                        \s*Settings\s+for\s+(?P<kernel_port>\w+):
                        \s+Supported\sports:\s+\[(?P<supp_ports>.*)\]
                        \s+Supported\slink\smodes:\s+(?P<supp_modes1>Not\s[Rr]eported|[0-9]+base[24a-zA-Z/_]+)
                        \s*(?P<supp_modes2>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes3>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes4>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes5>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes6>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes7>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes8>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes9>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes10>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes11>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes12>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes13>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes14>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<supp_modes15>[0-9]+base[24a-zA-Z/_]+)?
                        \s+Supported\spause\sframe\suse:\s+(?P<supp_pause>[a-zA-Z\- ]+)  #Symmetric Receive-only
                        \s+Supports\sauto-negotiation:\s+(?P<supp_autoneg>\w+)
                        (\s+Supported \sFEC\smodes:\s+(?P<supp_fec_modes>[0-9a-zA-Z /]+))?
                        (\s+(?P<supp_fec_modes2>[0-9a-zA-Z /]+))?
                        (\s+(?P<supp_fec_modes3>[0-9a-zA-Z /]+))?
                        \s+Advertised\slink\smodes:\s+(?P<adv_modes>Not\s[Rr]eported|[0-9]+base[24a-zA-Z/_]+) #Need 1000Mb/s and Not Reported
                        \s*(?P<adv_modes2>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes3>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes4>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes5>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes6>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes7>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes8>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes9>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes10>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes11>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes12>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes13>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes14>[0-9]+base[24a-zA-Z/_]+)?
                        \s*(?P<adv_modes15>[0-9]+base[24a-zA-Z/_]+)?
                        \s+Advertised\spause\sframe\suse:\s+(?P<adv_pause>\w+)
                        \s+Advertised\sauto-negotiation:\s+(?P<adv_autoneg>\w+)
                        (\s+Advertised\sFEC\smodes:\s+(?P<adv_fec_modes>[0-9a-zA-Z /]+))?
                        (\s+Link\spartner\sadvertised\slink\smodes:\s+(?P<lp_adv_modes>Not\s[Rr]eported|[0-9]+base[24a-zA-Z/-]+))?
                        \s*(?P<lp_adv_modes2>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes3>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes4>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes5>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes6>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes7>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes8>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes9>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes10>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes11>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes12>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes13>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes14>[0-9]+base[24a-zA-Z/-]+)?
                        \s*(?P<lp_adv_modes15>[0-9]+base[24a-zA-Z/-]+)?
                        (\s+Link\spartner\sadvertised\spause\sframe\suse:\s+(?P<lp_adv_pause>[a-zA-Z\- ]+))?
                        (\s+Link\spartner\sadvertised\sauto-negotiation:\s+(?P<lp_adv_autoneg>\w+))?
                        (\s+Link\spartner\sadvertised\sFEC\smodes:\s+(?P<lp_adv_fec_modes>[0-9a-zA-Z /]+))?
                        \s+Speed:\s+(?P<speed>[\w!\/]+)  # Need to pick up Unknown!
                        \s+Duplex:\s(?P<duplex>[\w!\/ \(\)]+)  # Need to pick up Unknown!
                        \s+Port:\s+(?P<port_type>[a-zA-Z ]+)
                        \s+PHYAD:\s(?P<phyad>\w+)
                        \s+Transceiver:\s(?P<transceiver>\w+)
                        \s+Auto-negotiation:\s(?P<autoneg>\w+)
                        (\s+MDI-X:\s(?P<mdix>[onfautUnkw \(\)]+))?  #Field not always here. on/off (auto), Unknown
                        (\s+Current\smessage\slevel:\s+(?P<curr_msg_lvl>[a-zA-Z0-9\(\)\-_\s]+))? #not always here in CL3
                        \s+Link\sdetected:\s+(?P<linkstate>\w+)\s*
                        ''', re.X)

    debug('Ethtool_out:\n' + output)
    match = pattern.match(output)
    if match:
        ethtool_dct = match.groupdict()
        debug('ethtool output in list format:' + pformat(match.group(0)))

    else:
        error('No ethtool match: {}'.format(output))
        return None

    return ethtool_dct
