"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
Parser for ethtool -m swpX
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "ethtool -m"


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None


    """
    # mlx: Cannot get Module EEPROM data: Invalid argument
    # bcm: Cannot get module EEPROM information
    no_eeprom_key = re.compile(r'''Cannot\sget\s[mM]odule\sEEPROM''')
    identifier_zero = re.compile(r'''0x00\s\(no\smodule\spresent,\sunknown,\sor\sunspecified\)''')

    ethtool_m_dct = {}
    for line in output.split('\n'):
        if no_eeprom_key.search(line) or identifier_zero.search(line):
            return None
        if not re.search('\w+',line):
            continue
        # debug(line)
        # debug('index is %s, value is %s' % (line.split(':')[0].strip(), line.split(':')[1].strip()))
        key = line.split(':', 1)[0].strip()
        value = line.split(':', 1)[1].strip()
        if key in ethtool_m_dct:
            # append - some interfaces have multiple lines of Transceiver type
            ethtool_m_dct[key] = ethtool_m_dct[key] + ' ; ' + value
        else:
            # Add
            ethtool_m_dct[key] = value

    debug('ethtool_m_dct:' + pformat(ethtool_m_dct))
    return ethtool_m_dct
