"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import logging
import re
from pprint import pformat

"""
Parser for ethtool -S swpX
"""

this_log = logging.getLogger(__name__)
debug = this_log.debug
info = this_log.info
warn = this_log.warning
error = this_log.error

name = "ethtool -S"


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    ethtool_S_dct = {}

    # Use regex named groups
    pattern=re.compile(r'''
NIC\s+statistics:
     \s+HwIfInOctets:\s+(?P<HwIfInOctets>\d+)
     \s+HwIfInUcastPkts:\s+(?P<HwIfInUcastPkts>\d+)
     \s+HwIfInBcastPkts:\s+(?P<HwIfInBcastPkts>\d+)
     \s+HwIfInMcastPkts:\s+(?P<HwIfInMcastPkts>\d+)
     \s+HwIfOutOctets:\s+(?P<HwIfOutOctets>\d+)
     \s+HwIfOutUcastPkts:\s+(?P<HwIfOutUcastPkts>\d+)
     \s+HwIfOutMcastPkts:\s+(?P<HwIfOutMcastPkts>\d+)
     \s+HwIfOutBcastPkts:\s+(?P<HwIfOutBcastPkts>\d+)
     \s+HwIfInDiscards:\s+(?P<HwIfInDiscards>\d+)
     \s+HwIfInL3Drops:\s+(?P<HwIfInL3Drops>\d+)
     \s+HwIfInBufferDrops:\s+(?P<HwIfInBufferDrops>\d+)
     \s+HwIfInAclDrops:\s+(?P<HwIfInAclDrops>\d+)
     (\s+HwIfInBlackholeDrops:\s+(?P<HwIfInBlackholeDrops>\d+))?
     \s+HwIfInDot3LengthErrors:\s+(?P<HwIfInDot3LengthErrors>\d+)
     \s+HwIfInErrors:\s+(?P<HwIfInErrors>\d+)
     (\s+HwIfInDot3FrameErrors:\s+(?P<HwIfInDot3FrameErrors>\d+))?
     \s+SoftInErrors:\s+(?P<SoftInErrors>\d+)
     \s+SoftInDrops:\s+(?P<SoftInDrops>\d+)
     \s+SoftInFrameErrors:\s+(?P<SoftInFrameErrors>\d+)
     \s+HwIfOutDiscards:\s+(?P<HwIfOutDiscards>\d+)
     \s+HwIfOutErrors:\s+(?P<HwIfOutErrors>\d+)
     \s+HwIfOutQDrops:\s+(?P<HwIfOutQDrops>\d+)
     \s+HwIfOutNonQDrops:\s+(?P<HwIfOutNonQDrops>\d+)
     \s+SoftOutErrors:\s+(?P<SoftOutErrors>\d+)
     \s+SoftOutDrops:\s+(?P<SoftOutDrops>\d+)
     \s+SoftOutTxFifoFull:\s+(?P<SoftOutTxFifoFull>\d+)
     \s+HwIfOutQLen:\s+(?P<HwIfOutQLen>\d+)
     (\s+HwIfInDot3FrameErrors:\s+(?P<HwIfInDot3FrameErrors1>\d+))?
     \s+HwIfInPausePkt:\s+(?P<HwIfInPausePkt>\d+)
     \s+HwIfOutPausePkt:\s+(?P<HwIfOutPausePkt>\d+)
     \s+HwIfInPfc0Pkt:\s+(?P<HwIfInPfc0Pkt>\d+)
     \s+HwIfOutPfc0Pkt:\s+(?P<HwIfOutPfc0Pkt>\d+)
     \s+HwIfInPfc1Pkt:\s+(?P<HwIfInPfc1Pkt>\d+)
     \s+HwIfOutPfc1Pkt:\s+(?P<HwIfOutPfc1Pkt>\d+)
     \s+HwIfInPfc2Pkt:\s+(?P<HwIfInPfc2Pkt>\d+)
     \s+HwIfOutPfc2Pkt:\s+(?P<HwIfOutPfc2Pkt>\d+)
     \s+HwIfInPfc3Pkt:\s+(?P<HwIfInPfc3Pkt>\d+)
     \s+HwIfOutPfc3Pkt:\s+(?P<HwIfOutPfc3Pkt>\d+)
     \s+HwIfInPfc4Pkt:\s+(?P<HwIfInPfc4Pkt>\d+)
     \s+HwIfOutPfc4Pkt:\s+(?P<HwIfOutPfc4Pkt>\d+)
     \s+HwIfInPfc5Pkt:\s+(?P<HwIfInPfc5Pkt>\d+)
     \s+HwIfOutPfc5Pkt:\s+(?P<HwIfOutPfc5Pkt>\d+)
     \s+HwIfInPfc6Pkt:\s+(?P<HwIfInPfc6Pkt>\d+)
     \s+HwIfOutPfc6Pkt:\s+(?P<HwIfOutPfc6Pkt>\d+)
     \s+HwIfInPfc7Pkt:\s+(?P<HwIfInPfc7Pkt>\d+)
     \s+HwIfOutPfc7Pkt:\s+(?P<HwIfOutPfc7Pkt>\d+)
                    ''', re.X)

    debug('ethtool_S_out:\n' + output)
    match = pattern.match(output)
    if match:
        debug('ethtool -S output in list format:' + pformat(match.group(0)))
        named_groups = ['HwIfInOctets', 'HwIfInUcastPkts', 'HwIfInBcastPkts',
                        'HwIfInMcastPkts', 'HwIfOutOctets', 'HwIfOutUcastPkts',
                        'HwIfOutMcastPkts', 'HwIfOutBcastPkts', 'HwIfInDiscards',
                        'HwIfInL3Drops', 'HwIfInBufferDrops', 'HwIfInAclDrops',
                        'HwIfInDot3LengthErrors', 'HwIfInErrors', 'SoftInErrors',
                        'SoftInDrops', 'SoftInFrameErrors', 'HwIfOutDiscards',
                        'HwIfOutErrors', 'HwIfOutQDrops', 'HwIfOutNonQDrops',
                        'SoftOutErrors', 'SoftOutDrops', 'SoftOutTxFifoFull',
                        'HwIfOutQLen', 'HwIfInDot3FrameErrors', 'HwIfInDot3FrameErrors1',
                        'HwIfInPausePkt', 'HwIfOutPausePkt',
                        'HwIfInPfc0Pkt', 'HwIfOutPfc0Pkt', 'HwIfInPfc1Pkt',
                        'HwIfOutPfc1Pkt', 'HwIfInPfc2Pkt', 'HwIfOutPfc2Pkt',
                        'HwIfInPfc3Pkt', 'HwIfOutPfc3Pkt', 'HwIfInPfc4Pkt',
                        'HwIfOutPfc4Pkt', 'HwIfInPfc5Pkt', 'HwIfOutPfc5Pkt',
                        'HwIfInPfc6Pkt', 'HwIfOutPfc6Pkt', 'HwIfInPfc7Pkt',
                        'HwIfOutPfc7Pkt']

        for name in named_groups:
            debug('Name %s matches: %s' % (name, match.group(name)))
            # Can't do a match.groupdict() since values need to be converted to type 'int'
            if match.group(name):
                ethtool_S_dct[name] = int(match.group(name))
            else:
                ethtool_S_dct[name] = 0

        ethtool_S_dct['HwIfInDot3FrameErrors'] += ethtool_S_dct['HwIfInDot3FrameErrors1']

    else:
        error('No ethtool -S match')
        return None

    return ethtool_S_dct
