"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
parser for ip link show
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "ip link show"


'''

Example:
root@mlx-2410-53:~# ip link show swp16

Link Up:
19: swp16: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 9216 qdisc pfifo_fast state UP mode DEFAULT group default qlen 1000
    link/ether 44:38:39:00:a8:33 brd ff:ff:ff:ff:ff:ff

Admin Down:
19: swp16: <BROADCAST,MULTICAST> mtu 9216 qdisc pfifo_fast state DOWN mode DEFAULT group default qlen 1000
    link/ether 44:38:39:00:a8:33 brd ff:ff:ff:ff:ff:ff

No Carrier:
55: swp52: <NO-CARRIER,BROADCAST,MULTICAST,UP> mtu 9216 qdisc pfifo_fast state DOWN mode DEFAULT group default qlen 1000
    link/ether 44:38:39:00:a8:16 brd ff:ff:ff:ff:ff:ff


'''


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    ansi_escape = re.compile(r'\x1B\[[0-?]*[ -/]*[@-~]')
    output = ansi_escape.sub('', output) 
 
    line_match = re.compile(r'''
                    [0-9]+:
                    \s(?P<dev>sw[0-9]*p[0-9]+(s[0-9]+)*):
                    \s<(?P<flags>.*)>\smtu
                    \s(?P<mtu>[0-9]+)\sqdisc
                    \s(?P<qdisc>.*)\sstate
                    \s(?P<state>UP|DOWN)\smode
                    \s(?P<mode>.*)\sgroup
                    \s(?P<group>.*)\sqlen
                    \s(?P<qlen>[0-9]+)\s+link/ether
                    \s(?P<hwaddr>.*)\sbrd
                    \s(?P<bcast>.*)$
                ''', re.X | re.MULTILINE)

    error_pattern = re.compile(r'\ADevice\ \"(.*)\"\ does\ not\ exist\.$', re.X)

    check_error = error_pattern.match(output.strip())

    if check_error:
        return None
    
    return dict([(f.groupdict().get("dev"), f.groupdict()) for f in re.finditer(line_match, output)])
