"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
parser for lldpctl
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "lldpctl"


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output

    """
    # TODO:    If parse failed, return None
    debug('Parsing lldpctl output table into a dictionary')
    debug('This may be deprecated by the lldpctl -f json or -f keyvalue output, which is easier parse')
    debug('the json output also handles multiple neighbors on the same interface')

    lldp_dct = {}
    new_port_flag = False
    interface_index = 0
    for line in output.split('\n'):
        debug(line)
        if re.search('unable\sto\sconnect\sto\ssocket', line):
            error('LLDP daemon is not started')
            break
        if new_port_flag:
            match = re.search('Interface:\s+(\w+), via: (\w+), RID: (\w+), Time: (.*)$', line)
            if match:
                debug('Create a new integer index, cant index by port since some port s have multiple neighbors')
                interface_index += 1
                lldp_dct[interface_index] = {}
                # Matches: Interface:    swp11s0, via: LLDP, RID: 13, Time: 10 days, 05:41:28
                debug('Match group: ' + match.group(0))
                lldp_dct[interface_index]['Interface'] = match.group(1)
                lldp_dct[interface_index]['Protocol'] = match.group(2)
                lldp_dct[interface_index]['RID'] = match.group(3)
                lldp_dct[interface_index]['Time'] = match.group(4).strip()
                new_port_flag = False
                continue
            elif re.search('LLDP neighbors:', line) or re.search('--------', line):
                debug('Throw away the first two lines of the output, do not clear new port flag yet')
                continue
            else:
                debug('Unexpected data in line, expected beginning of a new port')
                debug('Line is: ' + line)
                debug('Expected ---- or LLDP Neighbors:  or Interface: xx, via yy, RID: zz , TIME: aa')

        if re.search('--------', line):
            # -------------------------------------------------------------------------------
            debug('Matches: ----- ; new port in output, flag to create new key and continue')
            new_port_flag = True
            continue

        if re.search('Chassis:', line):
            debug('Chassis:     -skip this line')
            continue

        if re.search('Port:', line):
            debug('Port:        -skip this line')
            continue

        if re.search('/root/.bashrc:', line):
            debug('bashrc error -skip this line')
            continue

        # Rest of port data
        if re.search(':', line):
            debug('index is %s, value is %s' % (line.split(':', 1)[0].strip(), line.split(':', 1)[1].strip()))
            lldp_dct[interface_index][line.split(':', 1)[0].strip()] = line.split(':', 1)[1].strip()

    debug('lldp_dct: \n' + pformat(lldp_dct))

    return lldp_dct
