"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
parser for mlxlink
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "mlxlink"


'''
Alias shows all information about a port. Need to parse any individual part of this. 
alias mlx-link='mlxlink -d $(pcibus) -c -m -e --show_fec -p'

root@mlx-2410-53:~# mlx-link 22

Operational Info
----------------
State                           : Active
Physical state                  : LinkUp
Speed                           : 10GbE
Width                           : 1x
FEC                             : No FEC
Loopback Mode                   : No Loopback
Auto Negotiation                : ON

Supported Info
--------------
Enabled Link Speed              : 0x00001001 (10G,1G)
Supported Cable Speed           : 0x00001001 (10G,1G)

Troubleshooting Info
--------------------
Status Opcode                   : 0
Group Opcode                    : N/A
Recommendation                  : No issue was observed.

Physical Counters and BER Info
------------------------------
Time Since Last Clear [Min]     : 357.1
Effective Physical Errors       : 0
Raw Physical Errors Per Lane    : 0
Effective Physical BER          : 15E-255
Raw Physical BER                : 15E-255

EYE Opening Info
----------------
Physical Grade                  :  15186
Height Eye Opening [mV]         :    173
Phase  Eye Opening [psec]       :      0

FEC Capability Info
-------------------
FEC Capability 100GbE           : 0x0 (N/A)
FEC Capability 50GbE            : 0x0 (N/A)
FEC Capability 40GbE            : 0x0 (N/A)
FEC Capability 25GbE            : 0x7 (No-FEC,FireCode FEC,RS-FEC(528,514))
FEC Capability 10GbE            : 0x3 (No-FEC,FireCode FEC)

Module Info
-----------
Identifier                      : SFP28/SFP+
Compliance                      : Unspecified
Cable Technology                : Passive
Cable Type                      : Passive copper cable
OUI                             : Other
Vendor Name                     : FCM
Vendor Part Number              : QSFPx4-CU3M-CM
Vendor Serial Number            : MGD1J101189
Rev                             : 02
Attenuation (5g,7g,12g) [dB]    : 0,0,0
FW Version                      : N/A
Wavelength [nm]                 : N/A
Transfer Distance [m]           : 3
Digital Diagnostic Monitoring   : No
Power Class                     : 1.0 W max
CDR RX                          : OFF
CDR TX                          : OFF
LOS Alarm                       : N/A
Temperature [C]                 : 0 [0..0]
Voltage [mV]                    : 0 [0..0]
Bias Current [mA]               : 0 [0..0]
Rx Power Current [dBm]          : 0 [0..0]
Tx Power Current [dBm]          : 0 [0..0]



Link down example

root@mlx-2410-53:~# mlx-link 21

Operational Info
----------------
State                           : Polling
Physical state                  : ETH_AN_FSM_ABILITY_DETECT
Speed                           : N/A
Width                           : N/A
FEC                             : N/A
Loopback Mode                   : No Loopback
Auto Negotiation                : ON

Supported Info
--------------
Enabled Link Speed              : 0x00001001 (10G,1G)
Supported Cable Speed           : 0x00001001 (10G,1G)

Troubleshooting Info
--------------------
Status Opcode                   : 2
Group Opcode                    : PHY FW
Recommendation                  : Auto-negotiation no partner detected.

Physical Counters and BER Info
------------------------------
Time Since Last Clear [Min]     : N/A
Effective Physical Errors       : N/A
Raw Physical Errors Per Lane    : N/A
Effective Physical BER          : N/A
Raw Physical BER                : N/A

EYE Opening Info
----------------
Physical Grade                  : N/A
Height Eye Opening [mV]         : N/A
Phase  Eye Opening [psec]       : N/A

FEC Capability Info
-------------------
FEC Capability 100GbE           : 0x0 (N/A)
FEC Capability 50GbE            : 0x0 (N/A)
FEC Capability 40GbE            : 0x0 (N/A)
FEC Capability 25GbE            : 0x7 (No-FEC,FireCode FEC,RS-FEC(528,514))
FEC Capability 10GbE            : 0x3 (No-FEC,FireCode FEC)

Module Info
-----------
Identifier                      : SFP28/SFP+
Compliance                      : Unspecified
Cable Technology                : Passive
Cable Type                      : Passive copper cable
OUI                             : Other
Vendor Name                     : FCM
Vendor Part Number              : QSFPx4-CU3M-CM
Vendor Serial Number            : MGD1J101181
Rev                             : 02
Attenuation (5g,7g,12g) [dB]    : 0,0,0
FW Version                      : N/A
Wavelength [nm]                 : N/A
Transfer Distance [m]           : 3
Digital Diagnostic Monitoring   : No
Power Class                     : 1.0 W max
CDR RX                          : OFF
CDR TX                          : OFF
LOS Alarm                       : N/A
Temperature [C]                 : 0 [0..0]
Voltage [mV]                    : 0 [0..0]
Bias Current [mA]               : 0 [0..0]
Rx Power Current [dBm]          : 0 [0..0]
Tx Power Current [dBm]          : 0 [0..0]

Invalid Argument:
-E- Argument: -1 is invalid. It must be non-negative.

Invalid Port:
-E- Failed to find Local Port, please provide valid Port Number

BCM Box:
mlxlink: command not found

'''


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    # Check for failure due to on BCM asic
    bcm = re.compile(r'(command\snot\sfound)')
    if bcm.search(output):
        return None

    ansi_escape = re.compile(r'\x1B\[[0-?]*[ -/]*[@-~]')
    output = ansi_escape.sub('', output) 
 
    line_match = re.compile(r'(?P<key>.+):(?P<value>.*?)$', re.X)

    error_pattern = re.compile(r'-E-(.*)$', re.X)
    warning_pattern = re.compile(r'''^Warning:''')

    sep = re.compile(r'\A-+$', re.X)

    parsed_dct = {}

    check_error = error_pattern.match(output.strip())

    if check_error:
        return None

    for section in output.split('\n\n'):
        section = section.strip()
        dat = {}
        subheader_name = ""
        
        for line in section.split('\n'):
            if not line.strip():
                continue

            if warning_pattern.match(line):
                if 'ERRORS' not in parsed_dct:
                    error_count = 0
                    parsed_dct['ERRORS'] = {}
                error_count += 1
                parsed_dct['ERRORS'][str(error_count)] = line
                continue

            match = line_match.match(line)
            separator = sep.match(line)

            if match:
                k = match.group('key').strip()
                v = match.group('value').replace('\x00','').strip()
                debug('key: {}, value: {}'.format(k, v))
                dat[k] = v
            elif separator:
                subheader_name = subheader_candidate
            else:
                subheader_candidate = line.strip()
                debug('subheader name: {}'.format(subheader_candidate))

        parsed_dct[subheader_name] = dat

    return parsed_dct
