"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
Parser for net show system
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "net show system"


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """
    data_pattern = re.compile(r'^(?P<key>[a-zA-Z ]*)(\.)+(?P<value>.*)', re.X)

    parsed_dct = {}
    for line in output.split('\n'):
        if line.strip() == "":
            continue

        match = data_pattern.match(line)
        if match:
            dat = match.groupdict()
            parsed_dct.update({dat["key"]: dat["value"].strip()})
        else:
            print(line, "ERROR")
            error('Parser Bug: No Match found on line:')
            error(line)
            return None

    debug('net_show_system_dct:\n' + pformat(parsed_dct))
    if not parsed_dct:
        return None

    return parsed_dct
