"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
Parser for porttab
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "cat porttab"


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """
    hdr_pattern = re.compile(r'''^#.*$''')

    data_pattern = re.compile(r'''^
     \s*(?P<linux_intf>[swpfi0-9]+)
     \s+(?P<sdk_intf>[hgxce0-9]+)
     \s+(?P<unit>\d+)
     \s+(?P<is_fabric>\d+)
     (\s+(?P<logical_port>[-\d]+))?
     (\s+(?P<is_loopback>[\d]+))?
     (\s+(?P<is_internal>[\d]+))?
     \s*$''', re.X)

    debug('Create a porttab dict, where the port is the index and the rest of the line is a dct under it')
    parsed_dct = {}
    for line in output.split('\n'):
        if hdr_pattern.match(line):
            continue
        match = data_pattern.match(line)
        if match:
            port = match.group('linux_intf')
            if 'fp' in port:
                continue
            parsed_dct[port] = match.groupdict()
            parsed_dct[port].pop('linux_intf', 0)
        else:
            error('Parser Bug: No Match found on line:')
            error(line)
            return None

        # if line[0:3] == 'swp':
        #     debug('index is %s, value is %s' % (line.split('\t')[0].strip(), line.split('\t')[1].strip()))
        #     porttab_dct[line.split('\t')[0].strip()] = line.split('\t')[1].strip()

    debug('porttab_dct:\n' + pformat(parsed_dct))
    if not parsed_dct:
        return None

    return parsed_dct
