"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import logging
import re

"""
parser for ptmctl
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "ptmctl"


def parse(command, output):
    """
    ptmd_parser.  This is a quick and dirty parser that
    only parses the "cbl status"
    and keys it with the port name:
    {swp1: pass }
    There is a json output for ptmctl that makes a unique
    index key per port and contains all the fields
    """

    info('Parsing ptmctl output from dut')

    ptmd_dct = {}
    pattern = re.compile(r"""
                        (?P<port>[a-zA-Z0-9]+)
                        \s+(?P<status>[a-zA-Z/]+)
                        """, re.X)
    for line in output.split('\n'):
        if re.search('--------',line):
            continue

        if re.search('port\s+cbl',line):
            continue

        if re.search('status',line):
            continue

        # swp14  pass    N/A     N/A   N/A    N/A
        match = pattern.match(line)
        if match:
            debug('Match group: ' + match.group(0))
            debug('')
            ptmd_dct[match.group('port')] = match.group('status')
        else:
            error('Line did not match regex')
            error('Line: ' + line)
            return None

    return ptmd_dct


