"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
parser for topology.dot
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "cat topology.dot"
aliases = ['topology.dot']


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    '''
    "dell-s4000-10":"swp38" -- "qct-ly4r-04":"swp23" [LLDP="match_type=portdescr"]
    "dell-s4000-10":"swp49" -- "mlx-2700b-03":"swp5" [LLDP="match_type=portdescr"]
    "cumulus":"swp44" -- "qct-ly2-04":"swp20" [BFD="bfdtmpl=BFD1", LLDP="lldptmpl=LLDP1"]
    "dell-s6000-22":"swp20s0" -- "layer1-automation":"p72p2" [LLDP="match_type=portdescr"]
    '''
    parsed_dct = {}
    hdr_pattern = re.compile(r'''^\s*(graph.*|.*\})\s*$''')
    blank = re.compile(r'''^\s*$''')
    global_param_pattern = re.compile(r'''^\s*
            (?P<keyword>\w+)="?(?P<value>[\S]*)"?
            \s*$''', re.X)

    link_pattern = re.compile(r'''^\s*
            "?(?P<dut>[0-9a-zA-Z\-]+)"?:
            "?(?P<port>sw[0-9]*p[0-9s]+)"?
            \s+--\s+
            "?(?P<neighbor_dut>[0-9a-zA-Z\-]+)"?:
            "?(?P<neighbor_port>[a-zA-Z0-9]+)"?
            \s*(?P<port_param>\[[ ,="\w]+\])?
            \s*$''', re.X)
    # TODO create a list of all the params used on the link or global, and store with a key
    # For now, it will just be a string
    # port_param_pattern = re.compile(r'''(?P<keyword>\w+)="?(?P<value>)"?.*\])''')

    for line in output.split('\n'):
        # ignore the header and trailer:
        if hdr_pattern.match(line):
            debug('matched hdr')
            continue
        if blank.match(line):
            debug('blank line')
            continue
        global_param_match = global_param_pattern.match(line)
        if global_param_match:
            debug('Matched global_param')
            if 'global_param' not in parsed_dct:
                parsed_dct['global_param'] = {}
            parsed_dct['global_param'][global_param_match.group('keyword')] = global_param_match.group('value')
            continue

        # Done: What dict format do I want to return?
        # topology_dct[dut][port] = (neighbor_dut, neighbor_port)

        link_match = link_pattern.match(line)
        if link_match:
            debug('Matched link pattern')
            debug('Match: ' + pformat(link_match.group(0)))
            dut = link_match.group('dut')
            port = link_match.group('port')
            if dut not in parsed_dct:
                parsed_dct[dut] = {}
            parsed_dct[dut][port] = dict(neighbor_dut=link_match.group('neighbor_dut'),
                                         neighbor_port=link_match.group('neighbor_port'),
                                         port_param=link_match.group('port_param'))

        else:
            # make sure to error and return 'None' if any line doesn't match
            error('No {} parser match: {}'.format(name, line))
            return None

    return parsed_dct
