"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

"""
An example minimal module for a parser.
The module should contain a function named parse without the tests
 It may contain a name for the registry in the name variable.

Use debug logging to troubleshoot the parser.
Often you will also import re to handle regex'es
If parse fails, return 'None'
"""

import re
import logging
from pprint import pformat

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "change me"
aliases = ['change me']


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    pattern = re.compile(r'''^\s*
       (?P<list>\w+)
     \s+(?P<of>[a-z0-9\-]+)
    (\s+(?P<match>optionalParam))?
     \s+(?P<groups>\d+)
     \s*$''', re.X)

    # # if matching one line at a time:
    # for line in output.split('\n'):
    #     match = pattern.match(line)
    # # If need to ignore the header:
    #     if hdr_pattern.match(line):
    #         continue
    match = pattern.match(output)

    if match:
        debug('Match: ' + pformat(match.group(0)))
        # For each possible match group, create the dictionary keys = values
        parsed_dct = match.groupdict()
        # # If extracting a higher key from the table to create a hierarchy ( like 'port'),
        # # use this code instead.  Change 'higher_key' to be the actual key name:
        # higher_key = match.group('higher_key')
        # parsed_dct[higher_key] = match.groupdict()
        # # also remove that higher_key from the lower level:
        # parsed_dct[higher_key].pop('higher_key', 0)

    else:
        # make sure to error and return 'None' if any line doesn't match
        error('No {} parser match: {}'.format(name, output))
        return None

    return parsed_dct
